unit grfonts;

interface

uses
  DOS;

function getfontsize (fname : string) : integer;
function loadfont (fname : string) : boolean;
procedure unloadfont;
function getloadedfontname : string;
procedure drawchar (var dest; ltr : char; wx, wy : word; base : byte);

implementation

const
  font : pointer = nil;
  fontname : string = '';
  fontmem : integer = 0;

function getfontsize (fname : string) : integer;

var
  s : searchrec;

begin
  findfirst (fname,anyfile,s);
  if doserror=0 then
    getfontsize := s.size
  else
    getfontsize := -1;
end;

function loadfont (fname : string) : boolean;

var
  f : file;
  s : searchrec;
  count : word;
  p : pathstr;
  d : dirstr;
  n : namestr;
  e : extstr;
  i : integer;

  procedure freset (alloced : boolean);

  begin
    fontname := '';
    if alloced then
      freemem (font,fontmem);
    font := nil;
    fontmem := 0;
    loadfont := false;
  end;

begin
  if font<>nil then
    begin
      loadfont := false;
      exit;
    end;
  findfirst (fname,anyfile,s);
  if doserror<>0 then
    begin
      freset(false);
      exit;
    end;
  p := fexpand(fname);
  fsplit (p,d,n,e);
  fontname := n;
  fontmem := getfontsize(p);
  if fontmem=-1 then
    begin
      freset(false);
      exit;
    end;
  if memavail<fontmem then
    begin
      freset(false);
      exit;
    end;
    getmem (font,fontmem);
  assign (f,p);
  {$IFOPT I+}{$DEFINE IOPT}{$ENDIF}{$I-}
  reset (f,1);
  i := ioresult;
  if i<>0 then
    begin
      freset(true);
      exit;
    end;
  blockread (f,font^,fontmem,count);
  i := ioresult;
  if (i<>0) or (count<>fontmem) then
    begin
      close (f);
      i := ioresult;
      freset (true);
      exit;
    end;
  close (f);
  i := ioresult;
  {$IFDEF IOPT}{$I+}{$UNDEF IOPT}{$ENDIF}
  loadfont := true;
end;

procedure unloadfont;

begin
  if font=nil then
    exit;
  freemem(font,fontmem);
  font := nil;
  fontmem := 0;
  fontname := '';
end;

function getloadedfontname : string;

begin
  getloadedfontname := fontname;
end;

procedure drawchar (var dest; ltr : char; wx, wy : word; base : byte);
assembler;

{
  ds:si -- font source
  es:di -- screen destination
  ah    -- base value
  al    -- pixel xfer,
  ch    -- height count
  cl    -- width count
  bh    -- height max
  bl    -- width max
}

asm
  push  ds
  {load initial pointers}
  lds   si,[font]
  les   di,[dest]
  {get font height/width}
  mov   bx,word ptr ds:[si]
  add   si,2
  {determine offset for character in memory}
  mov   al,[ltr]
  imul  bl
  xor   cx,cx
  mov   cl,bh
  imul  cx
  add   si,ax
  {determine offset for destination memory}
  mov   cx,[wy]
  shl   cx,6
  add   di,cx
  shl   cx,2
  add   di,cx
  add   di,[wx]
  {get base}
  mov   ah,[base]
  cld
  mov   ch,bh
@drawloop:
  mov   cl,bl
  push  di
@drawrow:
  lodsb
  or    al,al
  jz    @trans
  add   al,ah
  mov   byte ptr es:[di],al
@trans:
  inc   di
  dec   cl
  jnz   @drawrow
  dec   ch
  jz    @Done
  pop   di
  add   di,320
  jmp   @drawloop
@Done:
  pop   di
  pop   ds
end;

begin
end.