unit multkeys;

{
 MULTKEYS.PAS -- Multiple keypress handling unit (for games, etc.)

 Written by Scott Earnest (sinykal@cyberspace.org)

 based on partial code in SWAG written by Lou DuChez

 Improvements made:

   - rolled into unit form
   - addeed constants for keys
   - added clean-up code so Ctrl-Break handler won't be necessary
   - corrected some scancode entries

 Notes:

   - Hardware is limited to detection of a maximum of 16 keys at a time.
     Most of the time that really isn't a problem, though.
   - Some constant definitions have the same value.  They are duplicated
     because 84-key keyboards and 101(+)-key keyboards use some different
     keys by produce identical results.
   - Print Scrn on a 101-keyboard sends both $2a and $37 (left shift +
     number pad asterisk).  This is the Print Scrn combination on 84-key
     keyboards.
}

interface

var
  keydown : array[0..127] of boolean;

const
  key_Esc        = $01;      key_1          = $02;
  key_2          = $03;      key_3          = $04;
  key_4          = $05;      key_5          = $06;
  key_6          = $07;      key_7          = $08;
  key_8          = $09;      key_9          = $0a;
  key_0          = $0b;      key_Minus      = $0c;
  key_Equals     = $0d;

  key_Backspace  = $0e;      key_Tab        = $0f;

  key_Q          = $10;      key_W          = $11;
  key_E          = $12;      key_R          = $13;
  key_T          = $14;      key_Y          = $15;
  key_U          = $16;      key_I          = $17;
  key_O          = $18;      key_P          = $19;
  key_LBracket   = $1a;      key_RBracket   = $1b;

  key_Enter      = $1c;      key_Control    = $1d;

  key_A          = $1e;      key_S          = $1f;
  key_D          = $20;      key_F          = $21;
  key_G          = $22;      key_H          = $23;
  key_J          = $24;      key_K          = $25;
  key_L          = $26;      key_Semicolon  = $27;
  key_Quote      = $28;      key_Tilde      = $29;

  key_LShift     = $2a;      key_Backslash  = $2b;

  key_Z          = $2c;      key_X          = $2d;
  key_C          = $2e;      key_V          = $2f;
  key_B          = $30;      key_N          = $31;
  key_M          = $32;      key_Comma      = $33;
  key_Period     = $34;      key_Slash      = $35;

  key_RShift     = $36;

  key_PrintScrn  = $37;      key_NPMult     = $37;

  key_Alt        = $38;      key_Space      = $39;
  key_CapsLock   = $3a;

  key_F1         = $3b;      key_F2         = $3c;
  key_F3         = $3d;      key_F4         = $3e;
  key_F5         = $3f;      key_F6         = $40;
  key_F7         = $41;      key_F8         = $42;
  key_F9         = $43;      key_F10        = $44;

  key_NumLock    = $45;      key_ScrollLock = $46;

  key_NP7        = $47;      key_NP8        = $48;
  key_NP9        = $49;      key_NPMinus    = $4a;
  key_NP4        = $4b;      key_NP5        = $4c;
  key_NP6        = $4d;      key_NPPlus     = $4e;
  key_NP1        = $4f;      key_NP2        = $50;
  key_NP3        = $51;      key_NP0        = $52;
  key_NPPeriod   = $53;

  key_Home       = $47;      key_UpArrow    = $48;
  key_PageUp     = $49;      key_LeftArrow  = $4b;
  key_RightArrow = $4d;      key_End        = $4f;
  key_DownArrow  = $50;      key_PageDown   = $51;

  key_Insert     = $52;      key_Delete     = $53;

  key_F11        = $57;      key_F12        = $58;

implementation

uses
  dos;

var
  exitchain : pointer;
  oldkeyhandler : procedure;

{$F+}

procedure multhandler; interrupt;

begin
  keydown[port[$60] mod 128] := (port[$60] < 128);
  asm pushf; end;
  oldkeyhandler;
  asm cli; end;
  memw[Seg0040:$001a] := memw[Seg0040:$001c];
  asm sti; end;
end;

procedure keyexit;

begin
  setintvec ($09,@oldkeyhandler);
  exitproc := exitchain;
end;

begin
  exitchain := exitproc;
  exitproc := @keyexit;
  getintvec($09,@oldkeyhandler);
  setintvec($09,@multhandler);
end.